<?php

/**
 * @file
 * Contains all local-compile checking functions.
 */

/**
 * Checks the availability of the local compilation.
 *
 * If the preferred method is set to local and the local compiler check fails,
 * an error message is displayed and the method is set to the second option.
 */
function closure_compiler_confirm_local($display_message = FALSE) {
  if (!closure_compiler_local_works() && variable_get('closure_compiler_process_method', 1) == 0) {
    if ($display_message) {
      drupal_set_message(t('Closure Compiler preferred method was set to use the API. Please check the errors with local compiling below.'), 'error');
    }
    else {
      // If we are not displaying the message, we are writing to watchdog.
      watchdog('closure_compiler', 'Error: Local compiling failed, switching to API mode.', WATCHDOG_ERROR);
    }
    variable_set('closure_compiler_process_method', 1);
  }
}

/**
 * Test that the local closure compiler function works correctly.
 */
function closure_compiler_local_works() {
  // If performed this check during this execution, do not test again.
  static $result;
  if (isset($result)) {
    return $result;
  }
  $test_file_path = CLOSURE_COMPILER_JS_PATH;
  if (!file_prepare_directory($test_file_path, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
    return FALSE;
  }
  $test_file_path .= '/test_js_file.js';
  // Try to create the file.
  $test_string = 'var i = 4; alert(i+i);';
  @file_put_contents($test_file_path, $test_string);
  // How are we going to compile locally if we can't write a file to JS folder?
  if (!file_exists($test_file_path)) {
    return FALSE;
  }
  $compiler_path = drupal_get_path('module', 'closure_compiler') . '/' . CLOSURE_COMPILER_FILE;
  // Pick simple optimizations for testing.
  $compilation_level = CLOSURE_COMPILER_SIMPLE;
  // Try to compile the file locally.
  $command = 'java -jar ' . $compiler_path . ' --compilation_level ' . $compilation_level . ' --js ' . $test_file_path;
  $compiled_string = shell_exec($command);
  if (is_file($test_file_path)) {
    // Clean up.
    file_unmanaged_delete($test_file_path);
  }
  else {
    return FALSE;
  }

  if (!$compiled_string || !strlen($compiled_string)) {
    return FALSE;
  }
  // So it should remove the whitespaces, resulting with a lower char count.
  $result = (strlen($compiled_string) < strlen($test_string));
  return $result;
}

/**
 * Tests whether compiling locally is operational.
 */
function closure_compiler_local_available() {
  return closure_compiler_java_installed() && closure_compiler_compiler_exists() && closure_compiler_local_works();
}

/**
 * Tests whether the local compiler is available.
 */
function closure_compiler_compiler_exists() {
  return file_exists(drupal_get_path('module', 'closure_compiler') . '/' . CLOSURE_COMPILER_FILE);
}

/**
 * Returns whether java is installed and executable on the machine.
 *
 * If there is a better way (I'm sure there is!), please let me know.
 */
function closure_compiler_java_installed() {
  $output = shell_exec('java');
  return (stripos($output, 'Usage') !== FALSE);
}
