<?php

/**
 * @file
 * Contains the actual compilation functions.
 */

/**
 * Main compilation function, run from the cron function.
 */
function closure_compiler_compile() {
  $js_dir = CLOSURE_COMPILER_JS_PATH;
  if (!file_prepare_directory($js_dir, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
    watchdog('closure_compiler', '%path directory not available', array('%path' => CLOSURE_COMPILER_JS_PATH), WATCHDOG_WARNING);
    return;
  }

  module_load_include('inc', 'closure_compiler', 'closure_compiler.functions');
  $js_files = closure_compiler_get_js_files();
  if (empty($js_files['unoptimized'])) {
    if (empty($js_files['optimized'])) {
      watchdog('closure_compiler', 'No JavaScript files found at %path', array('%path' => CLOSURE_COMPILER_JS_PATH), WATCHDOG_NOTICE);
    }
    else {
      watchdog('closure_compiler', 'No unoptimized JavaScript files found at %path', array('%path' => CLOSURE_COMPILER_JS_PATH), WATCHDOG_INFO);
    }
    return;
  }

  $process_limit = variable_get('closure_compiler_process_limit', 5);
  $compilation_level = variable_get('closure_compiler_compilation_level', CLOSURE_COMPILER_SIMPLE);
  $process_method = variable_get('closure_compiler_process_method', CLOSURE_COMPILER_SEND_CONTENTS);
  $closure_compiler_processed = variable_get('closure_compiler_processed', array());
  $succeeded = $failed = 0;
  foreach ($js_files['unoptimized'] as $uri => $js_file) {
    if ($process_limit && ($succeeded + $failed) == $process_limit) {
      break;
    }

    $file_statistics = array(
      'original_modification_time' => filemtime($uri),
      'original_js_size' => filesize($uri),
      'original_gz_size' => file_exists($uri . '.gz') ? filesize($uri . '.gz') : FALSE,
    );
    if (closure_compiler_compile_file($uri, $compilation_level, $process_method)) {
      $file_statistics['modification_time'] = filemtime($uri);
      $file_statistics['js_size'] = filesize($uri);
      $file_statistics['gz_size'] = file_exists($uri . '.gz') ? filesize($uri . '.gz') : FALSE;

      $closure_compiler_processed[$uri] = $file_statistics;
      ++$succeeded;
    }
    else {
      ++$failed;
    }
  }
  variable_set('closure_compiler_processed', $closure_compiler_processed);

  if (($succeeded + $failed) > 0) {
    watchdog('closure_compiler', 'Successfully compiled @succeeded javascript files, @failed failed.', array('@succeeded' => $succeeded, '@failed' => $failed), WATCHDOG_INFO);
  }
}

/**
 * Compiles a single JS file.
 *
 * @param string $uri
 *   The location of the JS file to compile.
 * @param string $compilation_level
 *   The compilation level to use (available options are defined).
 * @param int $process_method
 *   The process method to use (available options are defined).
 *
 * @return bool
 *   Whether the compilation was successful.
 */
function closure_compiler_compile_file($uri, $compilation_level, $process_method) {
  $result = FALSE;
  switch ($process_method) {
    case CLOSURE_COMPILER_LOCAL:
      $result = closure_compiler_compile_file_local($uri, $compilation_level);
      break;

    case CLOSURE_COMPILER_SEND_CONTENTS:
      if (filesize($uri) <= 200000) {
        $params['js_code'] = file_get_contents($uri);
      }
    case CLOSURE_COMPILER_SEND_URL:
      $params['code_url'] = file_create_url($uri);
      $params['compilation_level'] = $compilation_level;
      $result = closure_compiler_compile_file_api($uri, $params);
      break;
  }
  if ($result) {
    // Replace GZip-compressed file if necessary.
    if (variable_get('js_gzip_compression', TRUE) && variable_get('clean_url', 0) && extension_loaded('zlib')) {
      file_unmanaged_save_data(gzencode(file_get_contents($uri), 9, FORCE_GZIP), $uri . '.gz', FILE_EXISTS_REPLACE);
    }
    else if (file_exists($uri . '.gz')) {
      file_unmanaged_delete($uri . '.gz');
    }
  }
  return $result;
}

/**
 * Compiles a JS file locally.
 *
 * @param string $uri
 *   The location of the JS file to compile.
 * @param string $compilation_level
 *   The compilation level to use (available options are defined).
 *
 * @return bool
 *   Whether the compilation was successful.
 */
function closure_compiler_compile_file_local($uri, $compilation_level) {
  $compiler_path = drupal_get_path('module', 'closure_compiler') . '/' . CLOSURE_COMPILER_FILE;

  // The compiler script writes to a temporary file, prepends the signature
  // and replaces the original javascript file - the raw console output
  // can include error or warning messages, which breaks JS.
  $temp_file_path = drupal_tempnam(file_directory_temp(), 'jscc');
  $command = 'java -jar ' . $compiler_path . ' --compilation_level ' . $compilation_level . ' --js "' . drupal_realpath(check_plain($uri)) . '" --js_output_file ' . $temp_file_path;
  shell_exec($command);

  // Check whether the compiled file contains any content.
  if (!$compiled_js = file_get_contents($temp_file_path)) {
    return FALSE;
  }

  // Append signature to compiled script and overwrite the original file.
  if (!file_unmanaged_save_data($compiled_js, $uri, FILE_EXISTS_REPLACE)) {
    return FALSE;
  }

  // Cleanup.
  file_unmanaged_delete($temp_file_path);
  return TRUE;
}

/**
 * Compiles a JS file using the API.
 *
 * @param string $uri
 *   The location of the JS file to compile.
 * @param array $params
 *   Containing 'code_url' or 'js_code', and 'compilation_level'.
 *
 * @return bool
 *   Whether the compilation was successful.
 */
function closure_compiler_compile_file_api($uri, $params) {
  $params['output_info'] = 'compiled_code';
  $params['output_format'] = 'json';

  $response = drupal_http_request(CLOSURE_COMPILER_JS_OPTIMIZATION_URL, array(
    'headers' => array('Content-Type' => 'application/x-www-form-urlencoded'),
    'method' => 'POST',
    'data' => http_build_query($params, '', '&'),
  ));

  // Check the response is valid.
  if ($response->code != 200 || !$response->data) {
    watchdog('closure_compiler', 'Request error: @request', array('@request' => print_r($response, TRUE)));
    return FALSE;
  }

  // Decode json in the response.
  if (!($compiled_code = json_decode($response->data, TRUE)) || !is_array($compiled_code)) {
    watchdog('closure_compiler', 'Failed to parse JSON response: @response', array('@response' => $response->data));
    return FALSE;
  }

  // Check for server errors.
  if (array_key_exists('serverErrors', $compiled_code)) {
    watchdog('closure_compiler', 'Server error: @error', array('@error' => print_r($compiled_code['serverErrors'], TRUE)));
    return FALSE;
  }

  // Finally check the actual compiled code!
  if (array_key_exists('compiledCode', $compiled_code) && !empty($compiled_code['compiledCode'])) {
    // Success, write it back to the file.
    if (file_unmanaged_save_data($compiled_code['compiledCode'], $uri, FILE_EXISTS_REPLACE)) {
      return TRUE;
    }
    // Failed to write the compiled code back to the JS file.
    else {
      return FALSE;
    }
  }
  // The code is not present, or is empty.
  else {
    return FALSE;
  }
}
