<?php

/**
 * @file
 * Contains useful functions to be used elsewhere.
 */

/**
 * Returns an array containing all of the JS file URIs and modification times.
 */
function closure_compiler_get_js_files() {
  $closure_compiler_processed = variable_get('closure_compiler_processed', array());
  $js_files = array_merge(file_scan_directory(CLOSURE_COMPILER_JS_PATH, '/\.js$/', array('recurse' => FALSE)), $closure_compiler_processed);

  $optimized = array();
  $unoptimized = array();
  foreach ($js_files as $uri => $js_file) {
    // Check file still exists - if it doesn't, remove it from the variable.
    if (!file_exists($uri)) {
      unset($closure_compiler_processed[$uri]);
      continue;
    }
    if (!($modification_time = _closure_compiler_is_file_optimized($uri, $js_file))) {
      $unoptimized[$uri] = array(
        'original_js_size' => filesize($uri),
        'original_modification_time' => filemtime($uri),
        'original_gz_size' => file_exists($uri . '.gz') ? filesize($uri . '.gz') : FALSE,
      );
      unset($closure_compiler_processed[$uri]);
    }
    else {
      $optimized[$uri] = $js_file;
    }
  }
  variable_set('closure_compiler_processed', $closure_compiler_processed);

  return array(
    'optimized' => $optimized,
    'unoptimized' => $unoptimized,
  );
}

/**
 * Check if a JS file has already been optimized (internal).
 *
 * @param string $uri
 *   A valid path to a file.
 * @param array $js_file
 *   An associative array, containing information about the file.
 *
 * @return bool
 *   Whether the JS file is optimized.
 */
function _closure_compiler_is_file_optimized($uri, $js_file = array()) {
  // Ensure $js_file is an array, not stdObject
  $js_file = (array) $js_file;

  // If modification time is not in the $js_file array, it is unoptimized.
  if (!array_key_exists('modification_time', $js_file)) {
    return FALSE;
  }

  // If modification time is more recent, it needs to be re-optimized.
  $modification_time = filemtime($uri);
  if ($modification_time > $js_file['modification_time']) {
    return FALSE;
  }
  else {
    return TRUE;
  }
}

/**
 * Check if a JS file has already been optimized (external).
 *
 * We do not process the URI at all, so ensure you use the same path each time!
 *
 * @param string $uri
 *   A valid path to a file.
 *
 * @return bool
 *   Whether the JS file is optimized.
 */
function closure_compiler_is_file_optimized($uri) {
  $closure_compiler_processed = variable_get('closure_compiler_processed', array());

  // If the $uri is not in our processed list, it has not been processed.
  if (!array_key_exists($uri, $closure_compiler_processed)) {
    return FALSE;
  }

  return _closure_compiler_is_file_optimized($uri, $closure_compiler_processed[$uri]);
}
